* Generic session options - page and line size, no page breaks;
OPTIONS nonumber nodate nocenter pagesize=MAX linesize=120 formdlim=' ' spool;                                          
* Kill default titles on output, ODS tables in log, turn on graphics; 
TITLE; ODS TRACE OFF; *ODS LISTING; ODS GRAPHICS ON;

* To use FitTestG macro, run the code below;
* FitFewer = name of FitStatistics table for nested model;
* FitMore =  name of FitStatistics table for augmented model;
* ParmsFewer = name of OptInfo table for nested model;
* ParmsMore =  name of OptInfo table of augmented model;
%MACRO FitTestG(FitFewer, ParmsFewer, FitMore, ParmsMore);
DATA &ParmsFewer.; SET &ParmsFewer.; IF _N_=2 THEN CALL SYMPUT ("DFfewer",Value); RUN;
DATA &ParmsMore.;  SET &ParmsMore.;  IF _N_=2 THEN CALL SYMPUT ("DFmore",Value);  RUN;
DATA &FitFewer.;   SET &FitFewer.;   IF _N_=1 THEN CALL SYMPUT ("Devfewer",Value); RUN;
DATA &FitMore.;    SET &FitMore.;    IF _N_=1 THEN CALL SYMPUT ("Devmore",Value);  RUN;
DATA FitCompare; 
     DevFewer=&DevFewer.; DevMore=&DevMore.; DFfewer=&DFfewer.; DFmore=&DFmore.;
     TestStat=&Devfewer.-&Devmore.; TestDF=&DFmore.-&DFfewer.; 
     RegPvalue=1-PROBCHI(TestStat,TestDF);
     IF TestDF=1 THEN MixPvalue=(0.5*(1-PROBCHI(TestStat,TestDF)));
     IF TestDF=2 THEN MixPvalue=(0.5*(1-PROBCHI(TestStat,TestDF)))+
                                (0.5*(1-PROBCHI(TestStat,TestDF-1)));
     FORMAT RegPValue MixPValue pvalue6.4; RUN;
TITLE8 "Likelihood Ratio Test for &FitFewer. vs. &FitMore.";
TITLE9 "Regular p-value uses DF, mixture p-value uses DF=DF,DF-1";
PROC PRINT NOOBS DATA=FitCompare; RUN; TITLE8; TITLE9;
%MEND FitTestG;

********************************************************************************;
*******             BEGIN DATA MANIPULATION FOR EXAMPLE 3                *******;
*******             CHANGE "filesave" to your folder path                *******;
********************************************************************************;

* Location for original files for these models  change this path;
* \\Client\ precedes path in Virtual Desktop outside H drive;
%LET filesave=C:\Dropbox\23_PSQF6270\PSQF6270_Example3; 
LIBNAME filesave "&filesave.";

* Import Example 3 Excel data into work library;
PROC IMPORT DATAFILE="&filesave.\Excel_Example3.xlsx" OUT=work.Example3 DBMS=XLSX REPLACE; 
     GETNAMES=YES;
RUN;
* Create indicator-dummy-coded predictor variables; 
DATA work.Example3; SET work.Example3;  
  NvC=.; NvS=.;  * Make 2 new empty variables;
  IF ercond=1 THEN DO; NvC=0; NvS=0; END; * Recode if ercond=1=None;
  IF ercond=2 THEN DO; NvC=1; NvS=0; END; * Recode if ercond=2=Cognitive Reappraisal;
  IF ercond=3 THEN DO; NvC=0; NvS=1; END; * Recode if ercond=3=Suppression;
* Label all variables used in analysis;
  LABEL IPV=    "IPV: Inter-Partner Violence (0=N,1=Y)"
        ercond= "ercond: 1=None, 2=CogR, 3=Supp" 
        aggr=   "aggr: Aggressive Verbalizations"
        NvC=    "NvC: Condition None=0 vs CogR=1"
        NvS=    "NvS: Condition None=0 vs Supp=1"; 
* Filter to only cases complete on all variables to be used below;
  IF NMISS(aggr, IPV, ercond)>0 THEN DELETE;
RUN;


********************************************************************************;
******                 BEGIN EXAMPLE 3 COUNT OUTCOME MODELS              *******;
********************************************************************************;

* Open external file to save results to;
ODS RTF FILE="&filesave.\PSQF6270_Example3_SAS_Output.rtf" 
        STYLE=HTMLBlue STARTPAGE=NO BODYTITLE;

TITLE "SAS Cell Means for Aggressive Verbalizations Outcome";
PROC MEANS NONOBS NDEC=2 N MAX STDDEV MEAN STDERR DATA=work.Example3;
     CLASS ercond IPV; WAYS 2; VAR aggr; 
RUN; TITLE;

TITLE "SAS Histogram for Aggressive Verbalizations Outcome";
PROC UNIVARIATE NOPRINT DATA=work.Example3; VAR aggr; 
     HISTOGRAM aggr / MIDPOINTS=0 TO 24 BY 1; 
RUN; QUIT; TITLE;

TITLE "SAS Histogram for Aggressive Verbalizations Outcome by Cell";
PROC SGPANEL DATA=work.Example3; 
     PANELBY IPV ercond / ROWS=2 COLUMNS=3;
     HISTOGRAM aggr / BINSTART=0 BINWIDTH=1;
     LABEL IPV= "IPV" ercond="1=None, 2=CogR, 3=Supp";
RUN; QUIT; TITLE;


TITLE1 "SAS Link=Identity Dist=Normal Model using GLIMMIX";
TITLE2 "Using RSPL=OLS=REML to get SEs that match STATA and R";
PROC GLIMMIX DATA=work.Example3 GRADIENT NAMELEN=100 METHOD=RSPL PLOTS=(PEARSONPANEL);
MODEL aggr = IPV NvC NvS IPV*NvC IPV*NvS 
              / SOLUTION CHISQ DDFM=NONE LINK=IDENTITY DIST=NORMAL;
CONTRAST "DF=5 Multiv Wald Test of Model"  IPV 1, NvC 1, NvS 1, IPV*NvC 1, IPV*NvS 1 / CHISQ; 
CONTRAST "DF=2 Multiv Wald Test of Interaction"  IPV*NvC 1, IPV*NvS 1 / CHISQ; 
* Yhat cell means in original count model scale per combo of IPV and condition;
ESTIMATE "IPV=No,  Cond=None"  int 1 IPV 0 NvC 0 NvS 0 IPV*NvC 0 IPV*Nvs 0 / CL;
ESTIMATE "IPV=Yes, Cond=None"  int 1 IPV 1 NvC 0 NvS 0 IPV*NvC 0 IPV*Nvs 0 / CL;
ESTIMATE "IPV=No,  Cond=CogR"  int 1 IPV 0 NvC 1 NvS 0 IPV*NvC 0 IPV*Nvs 0 / CL;
ESTIMATE "IPV=Yes, Cond=CogR"  int 1 IPV 1 NvC 1 NvS 0 IPV*NvC 1 IPV*Nvs 0 / CL;
ESTIMATE "IPV=No,  Cond=Supp"  int 1 IPV 0 NvC 0 NvS 1 IPV*NvC 0 IPV*Nvs 0 / CL;
ESTIMATE "IPV=Yes, Cond=Supp"  int 1 IPV 1 NvC 0 NvS 1 IPV*NvC 0 IPV*Nvs 1 / CL;
* Simple slopes of IPV per condition;
ESTIMATE "No vs Yes IPV: None"    IPV 1 IPV*NvC 0 IPV*Nvs 0; 
ESTIMATE "No vs Yes IPV: CogR"    IPV 1 IPV*NvC 1 IPV*Nvs 0; 
ESTIMATE "No vs Yes IPV: Supp"    IPV 1 IPV*NvC 0 IPV*Nvs 1; 
* Simple slopes of condition per IPV;
ESTIMATE "None vs CogR: IPV=No"   NvC  1 NvS 0 IPV*NvC  0 IPV*Nvs 0; 
ESTIMATE "None vs Supp: IPV=No"   NvC  0 NvS 1 IPV*NvC  0 IPV*Nvs 0; 
ESTIMATE "CogR vs Supp: IPV=No"   NvC -1 NvS 1 IPV*NvC  0 IPV*Nvs 0; 
ESTIMATE "None vs CogR: IPV=Yes"  NvC  1 NvS 0 IPV*NvC  1 IPV*Nvs 0; 
ESTIMATE "None vs Supp: IPV=Yes"  NvC  0 NvS 1 IPV*NvC  0 IPV*Nvs 1; 
ESTIMATE "CogR vs Supp: IPV=Yes"  NvC -1 NvS 1 IPV*NvC -1 IPV*Nvs 1; 
* Simple slopes for interaction contrasts;
ESTIMATE "No/Yes IPV differ btw None/CogR"  IPV*NvC  1 IPV*Nvs 0; 
ESTIMATE "No/Yes IPV differ btw None/Supp"  IPV*NvC  0 IPV*Nvs 1; 
ESTIMATE "No/Yes IPV differ btw CogR/Supp"  IPV*NvC -1 IPV*Nvs 1; 
RUN; TITLE1; TITLE2;


* For demonstrating effect of changing link only -- not in handout;
TITLE1 "SAS Link=Log Dist=Normal Model using GLIMMIX";
TITLE2 "Using RSPL=OLS=REML to get SEs that match STATA and R";
PROC GLIMMIX DATA=work.Example3 GRADIENT NAMELEN=100 METHOD=RSPL;
MODEL aggr = IPV NvC NvS IPV*NvC IPV*NvS 
              / SOLUTION CHISQ DDFM=NONE LINK=LOG DIST=NORMAL;
CONTRAST "DF=5 Multiv Wald Test of Model"  IPV 1, NvC 1, NvS 1, IPV*NvC 1, IPV*NvS 1 / CHISQ; 
CONTRAST "DF=2 Multiv Wald Test of Interaction"  IPV*NvC 1, IPV*NvS 1 / CHISQ; 
* Yhat cell means in model and data scale per combination of IPV and condition;
ESTIMATE "IPV=No,  Cond=None"  int 1 IPV 0 NvC 0 NvS 0 IPV*NvC 0 IPV*Nvs 0 / ILINK CL;
ESTIMATE "IPV=Yes, Cond=None"  int 1 IPV 1 NvC 0 NvS 0 IPV*NvC 0 IPV*Nvs 0 / ILINK CL;
ESTIMATE "IPV=No,  Cond=CogR"  int 1 IPV 0 NvC 1 NvS 0 IPV*NvC 0 IPV*Nvs 0 / ILINK CL;
ESTIMATE "IPV=Yes, Cond=CogR"  int 1 IPV 1 NvC 1 NvS 0 IPV*NvC 1 IPV*Nvs 0 / ILINK CL;
ESTIMATE "IPV=No,  Cond=Supp"  int 1 IPV 0 NvC 0 NvS 1 IPV*NvC 0 IPV*Nvs 0 / ILINK CL;
ESTIMATE "IPV=Yes, Cond=Supp"  int 1 IPV 1 NvC 0 NvS 1 IPV*NvC 0 IPV*Nvs 1 / ILINK CL;
* Simple slopes of IPV per condition: EXP --> IRR effect sizes;
ESTIMATE "No vs Yes IPV: None"    IPV 1 IPV*NvC 0 IPV*Nvs 0 / EXP; 
ESTIMATE "No vs Yes IPV: CogR"    IPV 1 IPV*NvC 1 IPV*Nvs 0 / EXP; 
ESTIMATE "No vs Yes IPV: Supp"    IPV 1 IPV*NvC 0 IPV*Nvs 1 / EXP; 
* Simple slopes of condition per IPV: EXP --> IRR effect sizes;
ESTIMATE "None vs CogR: IPV=No"   NvC  1 NvS 0 IPV*NvC  0 IPV*Nvs 0 / EXP; 
ESTIMATE "None vs Supp: IPV=No"   NvC  0 NvS 1 IPV*NvC  0 IPV*Nvs 0 / EXP; 
ESTIMATE "CogR vs Supp: IPV=No"   NvC -1 NvS 1 IPV*NvC  0 IPV*Nvs 0 / EXP; 
ESTIMATE "None vs CogR: IPV=Yes"  NvC  1 NvS 0 IPV*NvC  1 IPV*Nvs 0 / EXP; 
ESTIMATE "None vs Supp: IPV=Yes"  NvC  0 NvS 1 IPV*NvC  0 IPV*Nvs 1 / EXP; 
ESTIMATE "CogR vs Supp: IPV=Yes"  NvC -1 NvS 1 IPV*NvC -1 IPV*Nvs 1 / EXP; 
* Simple slopes for interaction contrasts: EXP --> IRR effect sizes;
ESTIMATE "No/Yes IPV differ btw None/CogR"  IPV*NvC  1 IPV*Nvs 0 / EXP; 
ESTIMATE "No/Yes IPV differ btw None/Supp"  IPV*NvC  0 IPV*Nvs 1 / EXP; 
ESTIMATE "No/Yes IPV differ btw CogR/Supp"  IPV*NvC -1 IPV*Nvs 1 / EXP; 
RUN; TITLE1; TITLE2;


TITLE1 "SAS Link=Log Dist=Poisson Model using GLIMMIX";
TITLE2 "Using RSPL=OLS=REML to get SEs that match STATA and R";
PROC GLIMMIX DATA=work.Example3 GRADIENT NAMELEN=100 METHOD=RSPL;
MODEL aggr = IPV NvC NvS IPV*NvC IPV*NvS 
              / SOLUTION CHISQ DDFM=NONE LINK=LOG DIST=POISSON;
CONTRAST "DF=5 Multiv Wald Test of Model"  IPV 1, NvC 1, NvS 1, IPV*NvC 1, IPV*NvS 1 / CHISQ; 
CONTRAST "DF=2 Multiv Wald Test of Interaction"  IPV*NvC 1, IPV*NvS 1 / CHISQ; 
* Yhat cell means in model and data scale per combination of IPV and condition;
ESTIMATE "IPV=No,  Cond=None"  int 1 IPV 0 NvC 0 NvS 0 IPV*NvC 0 IPV*Nvs 0 / ILINK CL;
ESTIMATE "IPV=Yes, Cond=None"  int 1 IPV 1 NvC 0 NvS 0 IPV*NvC 0 IPV*Nvs 0 / ILINK CL;
ESTIMATE "IPV=No,  Cond=CogR"  int 1 IPV 0 NvC 1 NvS 0 IPV*NvC 0 IPV*Nvs 0 / ILINK CL;
ESTIMATE "IPV=Yes, Cond=CogR"  int 1 IPV 1 NvC 1 NvS 0 IPV*NvC 1 IPV*Nvs 0 / ILINK CL;
ESTIMATE "IPV=No,  Cond=Supp"  int 1 IPV 0 NvC 0 NvS 1 IPV*NvC 0 IPV*Nvs 0 / ILINK CL;
ESTIMATE "IPV=Yes, Cond=Supp"  int 1 IPV 1 NvC 0 NvS 1 IPV*NvC 0 IPV*Nvs 1 / ILINK CL;
* Simple slopes of IPV per condition: EXP --> IRR effect sizes;
ESTIMATE "No vs Yes IPV: None"    IPV 1 IPV*NvC 0 IPV*Nvs 0 / EXP; 
ESTIMATE "No vs Yes IPV: CogR"    IPV 1 IPV*NvC 1 IPV*Nvs 0 / EXP; 
ESTIMATE "No vs Yes IPV: Supp"    IPV 1 IPV*NvC 0 IPV*Nvs 1 / EXP; 
* Simple slopes of condition per IPV: EXP --> IRR effect sizes;
ESTIMATE "None vs CogR: IPV=No"   NvC  1 NvS 0 IPV*NvC  0 IPV*Nvs 0 / EXP; 
ESTIMATE "None vs Supp: IPV=No"   NvC  0 NvS 1 IPV*NvC  0 IPV*Nvs 0 / EXP; 
ESTIMATE "CogR vs Supp: IPV=No"   NvC -1 NvS 1 IPV*NvC  0 IPV*Nvs 0 / EXP; 
ESTIMATE "None vs CogR: IPV=Yes"  NvC  1 NvS 0 IPV*NvC  1 IPV*Nvs 0 / EXP; 
ESTIMATE "None vs Supp: IPV=Yes"  NvC  0 NvS 1 IPV*NvC  0 IPV*Nvs 1 / EXP; 
ESTIMATE "CogR vs Supp: IPV=Yes"  NvC -1 NvS 1 IPV*NvC -1 IPV*Nvs 1 / EXP; 
* Simple slopes for interaction contrasts: EXP --> IRR effect sizes;
ESTIMATE "No/Yes IPV differ btw None/CogR"  IPV*NvC  1 IPV*Nvs 0 / EXP; 
ESTIMATE "No/Yes IPV differ btw None/Supp"  IPV*NvC  0 IPV*Nvs 1 / EXP; 
ESTIMATE "No/Yes IPV differ btw CogR/Supp"  IPV*NvC -1 IPV*Nvs 1 / EXP; 
* Save tables with #parms and -2LL for FitTestG macro;
ODS OUTPUT OptInfo=ParmsP FitStatistics=FitP;
RUN; TITLE1; TITLE2;


TITLE1 "SAS Link=Log Dist=Negative Binomial Model using GLIMMIX";
TITLE2 "Using RSPL=OLS=REML to get SEs that match STATA and R";
PROC GLIMMIX DATA=work.Example3 GRADIENT NAMELEN=100 METHOD=RSPL;
MODEL aggr = IPV NvC NvS IPV*NvC IPV*NvS 
              / SOLUTION CHISQ DDFM=NONE LINK=LOG DIST=NEGBIN;
CONTRAST "DF=5 Multiv Wald Test of Model"  IPV 1, NvC 1, NvS 1, IPV*NvC 1, IPV*NvS 1 / CHISQ; 
CONTRAST "DF=2 Multiv Wald Test of Interaction"  IPV*NvC 1, IPV*NvS 1 / CHISQ; 
* Yhat cell means in model and data scale per combination of IPV and condition;
ESTIMATE "IPV=No,  Cond=None"  int 1 IPV 0 NvC 0 NvS 0 IPV*NvC 0 IPV*Nvs 0 / ILINK CL;
ESTIMATE "IPV=Yes, Cond=None"  int 1 IPV 1 NvC 0 NvS 0 IPV*NvC 0 IPV*Nvs 0 / ILINK CL;
ESTIMATE "IPV=No,  Cond=CogR"  int 1 IPV 0 NvC 1 NvS 0 IPV*NvC 0 IPV*Nvs 0 / ILINK CL;
ESTIMATE "IPV=Yes, Cond=CogR"  int 1 IPV 1 NvC 1 NvS 0 IPV*NvC 1 IPV*Nvs 0 / ILINK CL;
ESTIMATE "IPV=No,  Cond=Supp"  int 1 IPV 0 NvC 0 NvS 1 IPV*NvC 0 IPV*Nvs 0 / ILINK CL;
ESTIMATE "IPV=Yes, Cond=Supp"  int 1 IPV 1 NvC 0 NvS 1 IPV*NvC 0 IPV*Nvs 1 / ILINK CL;
* Simple slopes of IPV per condition: EXP --> IRR effect sizes;
ESTIMATE "No vs Yes IPV: None"    IPV 1 IPV*NvC 0 IPV*Nvs 0 / EXP; 
ESTIMATE "No vs Yes IPV: CogR"    IPV 1 IPV*NvC 1 IPV*Nvs 0 / EXP; 
ESTIMATE "No vs Yes IPV: Supp"    IPV 1 IPV*NvC 0 IPV*Nvs 1 / EXP; 
* Simple slopes of condition per IPV: EXP --> IRR effect sizes;
ESTIMATE "None vs CogR: IPV=No"   NvC  1 NvS 0 IPV*NvC  0 IPV*Nvs 0 / EXP; 
ESTIMATE "None vs Supp: IPV=No"   NvC  0 NvS 1 IPV*NvC  0 IPV*Nvs 0 / EXP; 
ESTIMATE "CogR vs Supp: IPV=No"   NvC -1 NvS 1 IPV*NvC  0 IPV*Nvs 0 / EXP; 
ESTIMATE "None vs CogR: IPV=Yes"  NvC  1 NvS 0 IPV*NvC  1 IPV*Nvs 0 / EXP; 
ESTIMATE "None vs Supp: IPV=Yes"  NvC  0 NvS 1 IPV*NvC  0 IPV*Nvs 1 / EXP; 
ESTIMATE "CogR vs Supp: IPV=Yes"  NvC -1 NvS 1 IPV*NvC -1 IPV*Nvs 1 / EXP; 
* Simple slopes for interaction contrasts: EXP --> IRR effect sizes;
ESTIMATE "No/Yes IPV differ btw None/CogR"  IPV*NvC  1 IPV*Nvs 0 / EXP; 
ESTIMATE "No/Yes IPV differ btw None/Supp"  IPV*NvC  0 IPV*Nvs 1 / EXP; 
ESTIMATE "No/Yes IPV differ btw CogR/Supp"  IPV*NvC -1 IPV*Nvs 1 / EXP; 
* Save tables with #parms and -2LL for FitTestG macro;
ODS OUTPUT OptInfo=ParmsNB FitStatistics=FitNB;
* Save predicted counts per real person to dataset;
OUTPUT OUT=work.NegBinPred PREDICTED(ILINK)=PredCount; 
RUN; TITLE1; TITLE2;

* Call FitTestG macro to compute LRT between Poisson and NegBin;
%FitTestG(FitFewer=FitP, ParmsFewer=ParmsP, FitMore=FitNB, ParmsMore=ParmsNB);

* Save corr of pred count with aggr to square as R2;
TITLE "Correlation of Predicted and Actual Count";
PROC CORR NOSIMPLE DATA=work.NegBinPred OUT=work.Rpred; 
     VAR aggr; WITH PredCount; RUN; 
* Compute R2 in saved output;
DATA work.Rpred; SET work.Rpred;
     WHERE _TYPE_="CORR"; R2=aggr*aggr; RUN;
TITLE "R2 of Predicted and Actual Count";
PROC PRINT NOOBS DATA=work.Rpred; VAR R2; RUN; TITLE;


TITLE1 "SAS Zero-Inflated Poisson Model using GENDMOD";
TITLE2 "Only intercept in zero-inflation model (predict logit of extra zero)";
PROC GENMOD DATA=work.Example3 NAMELEN=100;
MODEL aggr = IPV NvC NvS IPV*NvC IPV*NvS / LINK=LOG DIST=ZIP;
ZEROMODEL / LINK=LOGIT; 
ESTIMATE "Extra zero model intercept" @ZERO intercept 1 / EXP;
CONTRAST "DF=5 Multiv Wald Test of Model"  IPV 1, NvC 1, NvS 1, IPV*NvC 1, IPV*NvS 1 / WALD; 
CONTRAST "DF=2 Multiv Wald Test of Interaction"  IPV*NvC 1, IPV*NvS 1 / WALD; 
RUN; TITLE1; TITLE2;


TITLE1 "SAS Zero-Inflated Negative Binomial Model using GENMOD";
TITLE2 "Only intercept in zero-inflation model (predicted in logits)";
PROC GENMOD DATA=work.Example3 NAMELEN=100;
MODEL aggr = IPV NvC NvS IPV*NvC IPV*NvS / LINK=LOG DIST=ZINB;
ZEROMODEL / LINK=LOGIT; 
ESTIMATE "Extra zero model intercept" @ZERO intercept 1 / EXP;
CONTRAST "DF=5 Multiv Wald Test of Model"  IPV 1, NvC 1, NvS 1, IPV*NvC 1, IPV*NvS 1 / WALD; 
CONTRAST "DF=2 Multiv Wald Test of Interaction"  IPV*NvC 1, IPV*NvS 1 / WALD; 
RUN; TITLE1; TITLE2;


TITLE "SAS Heterogeneous Negative Binomial Model using NLMIXED";
PROC NLMIXED DATA=work.Example3 METHOD=GAUSS TECH=QUANEW GCONV=1e-12;
* Must list all parms to be estimated here with start values (from Stata here);
* Bs = fixed effects predicting the mean;
* Ds = fixed effects predicting the dispersion (scale factor);
PARMS B0 0.99 B1IPV  0.11 B2NvC -1.08 B3NvS -0.14 B4IPVxNvC  -1.93 B5IPVxNvS 0.52
      D0 0.48 D1IPV -0.81 D2NvC  0.64 D3NvS  0.05 D4IPVxNvC -11.95 D5IPVxNvS 0.60;
 * Linear predictor for the log count mean;
   LogCount = B0 + B1IPV*IPV + B2NvC*NvC + B3NvS*NvS 
               + B4IPVxNvC*IPV*NvC + B5IPVxNvS*IPV*NvS;
   ExpCount = EXP(LogCount); * Inverse link;
 * Linear predictor for the log of the dispersion scale parameter;
   LogScale = D0 + D1IPV*IPV + D2NvC*NvC + D3NvS*NvS 
               + D4IPVxNvC*IPV*NvC + D5IPVxNvS*IPV*NvS;
   ExpScale=EXP(LogScale); * Inverse Link;
 * Log-likelihood for NB-1 (constant) or NB-2 (multiplicative);
   * NB-1: LL = aggr*LOG(ExpScale) - (aggr+(1/(ExpScale/ExpCount)))*LOG(1+ExpScale)
                 + lgamma(aggr+(1/(ExpScale/ExpCount))) - lgamma(1/(ExpScale/ExpCount)) - lgamma(aggr+1);
   * NB-2; LL = aggr*LOG(ExpScale*ExpCount) - (aggr+(1/ExpScale))*LOG(1+(ExpScale*ExpCount))
                 + lgamma(aggr+(1/ExpScale)) - lgamma(1/ExpScale) - lgamma(aggr+1);
MODEL aggr ~ GENERAL(LL);
RUN;


* Close external file of output;
ODS RTF CLOSE;
