// Prevent "more" messages from appearing
set more off
// Control line length
set linesize 150

********************************************************************************
*******             BEGIN DATA MANIPULATION FOR EXAMPLE 3                *******
*******             CHANGE "filesave" to your folder path                *******
********************************************************************************

// Defining global variable for file location to be replaced in code below
// \\Client\ precedes path in Virtual Desktop outside H drive;
global filesave "C:\Dropbox\23_PSQF6270\PSQF6270_Example3"

// Import Example 3 Excel data
clear // clear memory in case a dataset is already open
import excel "$filesave\Excel_Example3.xlsx", firstrow case(preserve) clear 

// STATA code to create indicator-dummy-coded predictor variables
gen NvC=.  // Make 2 new empty variables
gen NvS=.
replace NvC=0 if ercond==1  // Replace if ercond=1=None
replace NvS=0 if ercond==1
replace NvC=1 if ercond==2  // Replace if ercond=2=CogR
replace NvS=0 if ercond==2
replace NvC=0 if ercond==3  // Replace if ercond=3=Supp
replace NvS=1 if ercond==3
label variable IPV    "IPV: Inter-Partner Violence (0=N,1=Y)"
label variable ercond "ercond: 1=None, 2=CogR, 3=Supp"
label variable aggr   "aggr: Aggressive Verbalizations"
label variable NvC    "NvC: Condition None=0 vs. CogR=1"
label variable NvS    "NvS: Condition None=0 vs. Supp=1"
// Filter to only cases complete on all variables to be used below
egen nummiss = rowmiss(aggr IPV ercond)
drop if nummiss>0


********************************************************************************
******             BEGIN EXAMPLE 3 COUNT OUTCOME MODELS                  *******
********************************************************************************

// Open external file to save results to
log using $filesave\PSQF6270_Example3_STATA_Output.log, replace name(Example3)

display "STATA Cell Means for Aggressive Verbalizations"
bysort ercond IPV: tabstat aggr, statistics(n max sd mean semean)
 
display "STATA Histogram for Aggressive Verbalizations"
hist aggr, percent normal discrete width(1) start(0) 
graph export "$filesave\STATA Overall Histogram.png", replace

display "STATA Histogram for Aggressive Verbalizations by Cell"
hist aggr, by(IPV ercond) percent normal discrete width(1) start(0) 
graph export "$filesave\STATA By Cell Histogram.png", replace
      
display "STATA Link=Identity Dist=Normal Model using glm"
display "SEs use REML=OLS instead for Comparability with R"
glm aggr c.IPV c.NvC c.NvS c.IPV#c.NvC c.IPV#c.NvS, ml link(identity) family(gaussian) nolog
display "-2LL= " e(ll)*-2  // Print -2LL for model
// DF=5 Multiv Wald Test of Model
test (c.IPV=0)(c.NvC=0)(c.NvS=0)(c.IPV#c.NvC=0)(c.IPV#c.NvS=0)
// DF=2 Multiv Wald Test of Interaction
test (c.IPV#c.NvC=0)(c.IPV#c.NvS=0)
// Yhat cell means in original count model scale per condition
margins, at(c.IPV=(0(1)1) c.NvC=0 c.NvS=0) predict(xb) // None
margins, at(c.IPV=(0(1)1) c.NvC=1 c.NvS=0) predict(xb) // CogR
margins, at(c.IPV=(0(1)1) c.NvC=0 c.NvS=1) predict(xb) // Supp
// Simple slopes of IPV per condition
lincom c.IPV*1 + c.IPV#c.NvC*0 + c.IPV#c.NvS*0  // No vs Yes IPV: None
lincom c.IPV*1 + c.IPV#c.NvC*1 + c.IPV#c.NvS*0  // No vs Yes IPV: CogR
lincom c.IPV*1 + c.IPV#c.NvC*0 + c.IPV#c.NvS*1  // No vs Yes IPV: Supp
// Simple slopes of condition per IPV
lincom c.NvC*1  + c.NvS*0 + c.IPV#c.NvC*0  + c.IPV#c.NvS*0  // None vs CogR: IPV=No
lincom c.NvC*0  + c.NvS*1 + c.IPV#c.NvC*0  + c.IPV#c.NvS*0  // None vs Supp: IPV=No
lincom c.NvC*-1 + c.NvS*1 + c.IPV#c.NvC*0  + c.IPV#c.NvS*0  // CogR vs Supp: IPV=No
lincom c.NvC*1  + c.NvS*0 + c.IPV#c.NvC*1  + c.IPV#c.NvS*0  // None vs CogR: IPV=Yes
lincom c.NvC*0  + c.NvS*1 + c.IPV#c.NvC*0  + c.IPV#c.NvS*1  // None vs Supp: IPV=Yes
lincom c.NvC*-1 + c.NvS*1 + c.IPV#c.NvC*-1 + c.IPV#c.NvS*1  // CogR vs Supp: IPV=Yes
// Simple slopes for interaction contrasts
lincom c.IPV#c.NvC*1  + c.IPV#c.NvS*0  // No/Yes IPV differ btw None/CogR
lincom c.IPV#c.NvC*0  + c.IPV#c.NvS*1  // No/Yes IPV differ btw None/Supp
lincom c.IPV#c.NvC*-1 + c.IPV#c.NvS*1  // No/Yes IPV differ btw CogR/Supp


display "STATA Link=Log Dist=Poisson Model using glm"
glm aggr c.IPV c.NvC c.NvS c.IPV#c.NvC c.IPV#c.NvS, ml link(log) family(poisson) nolog
display "-2LL= " e(ll)*-2  // Print -2LL for model 
// DF=5 Multiv Wald Test of Model R2
test (c.IPV=0)(c.NvC=0)(c.NvS=0)(c.IPV#c.NvC=0)(c.IPV#c.NvS=0)
// DF=2 Multiv Wald Test of Interaction
test (c.IPV#c.NvC=0)(c.IPV#c.NvS=0)
// Yhat cell means in log count model scale per condition
margins, at(c.IPV=(0(1)1) c.NvC=0 c.NvS=0) predict(xb) // None
margins, at(c.IPV=(0(1)1) c.NvC=1 c.NvS=0) predict(xb) // CogR
margins, at(c.IPV=(0(1)1) c.NvC=0 c.NvS=1) predict(xb) // Supp
// Y hat cell means in expected count data scale per ercond
margins, at(c.IPV=(0(1)1) c.NvC=0 c.NvS=0) // None
margins, at(c.IPV=(0(1)1) c.NvC=1 c.NvS=0) // CogR
margins, at(c.IPV=(0(1)1) c.NvC=0 c.NvS=1) // Supp
// Simple slopes of IPV per condition
lincom c.IPV*1 + c.IPV#c.NvC*0 + c.IPV#c.NvS*0  // No vs Yes IPV: None
lincom c.IPV*1 + c.IPV#c.NvC*1 + c.IPV#c.NvS*0  // No vs Yes IPV: CogR
lincom c.IPV*1 + c.IPV#c.NvC*0 + c.IPV#c.NvS*1  // No vs Yes IPV: Supp
// Simple slopes of condition per IPV
lincom c.NvC*1  + c.NvS*0 + c.IPV#c.NvC*0  + c.IPV#c.NvS*0  // None vs CogR: IPV=No
lincom c.NvC*0  + c.NvS*1 + c.IPV#c.NvC*0  + c.IPV#c.NvS*0  // None vs Supp: IPV=No
lincom c.NvC*-1 + c.NvS*1 + c.IPV#c.NvC*0  + c.IPV#c.NvS*0  // CogR vs Supp: IPV=No
lincom c.NvC*1  + c.NvS*0 + c.IPV#c.NvC*1  + c.IPV#c.NvS*0  // None vs CogR: IPV=Yes
lincom c.NvC*0  + c.NvS*1 + c.IPV#c.NvC*0  + c.IPV#c.NvS*1  // None vs Supp: IPV=Yes
lincom c.NvC*-1 + c.NvS*1 + c.IPV#c.NvC*-1 + c.IPV#c.NvS*1  // CogR vs Supp: IPV=Yes
// Simple slopes for interaction contrasts
lincom c.IPV#c.NvC*1  + c.IPV#c.NvS*0  // No/Yes IPV differ btw None/CogR
lincom c.IPV#c.NvC*0  + c.IPV#c.NvS*1  // No/Yes IPV differ btw None/Supp
lincom c.IPV#c.NvC*-1 + c.IPV#c.NvS*1  // No/Yes IPV differ btw CogR/Supp
           
 
display "STATA Link=Log Dist=Poisson Model using glm"
display "Request Incidence-Rate Ratios (via eform or irr)"
glm aggr c.IPV c.NvC c.NvS c.IPV#c.NvC c.IPV#c.NvS, eform ml link(log) family(poisson) nolog
// Simple slopes of IPV per condition
lincom c.IPV*1 + c.IPV#c.NvC*0 + c.IPV#c.NvS*0, irr  // No vs Yes IPV: None
lincom c.IPV*1 + c.IPV#c.NvC*1 + c.IPV#c.NvS*0, irr  // No vs Yes IPV: CogR
lincom c.IPV*1 + c.IPV#c.NvC*0 + c.IPV#c.NvS*1, irr  // No vs Yes IPV: Supp
// Simple slopes of condition per IPV
lincom c.NvC*1  + c.NvS*0 + c.IPV#c.NvC*0  + c.IPV#c.NvS*0, irr  // None vs CogR: IPV=No
lincom c.NvC*0  + c.NvS*1 + c.IPV#c.NvC*0  + c.IPV#c.NvS*0, irr  // None vs Supp: IPV=No
lincom c.NvC*-1 + c.NvS*1 + c.IPV#c.NvC*0  + c.IPV#c.NvS*0, irr  // CogR vs Supp: IPV=No
lincom c.NvC*1  + c.NvS*0 + c.IPV#c.NvC*1  + c.IPV#c.NvS*0, irr  // None vs CogR: IPV=Yes
lincom c.NvC*0  + c.NvS*1 + c.IPV#c.NvC*0  + c.IPV#c.NvS*1, irr  // None vs Supp: IPV=Yes
lincom c.NvC*-1 + c.NvS*1 + c.IPV#c.NvC*-1 + c.IPV#c.NvS*1, irr  // CogR vs Supp: IPV=Yes
// Simple slopes for interaction contrasts
lincom c.IPV#c.NvC*1  + c.IPV#c.NvS*0, irr  // No/Yes IPV differ btw None/CogR
lincom c.IPV#c.NvC*0  + c.IPV#c.NvS*1, irr  // No/Yes IPV differ btw None/Supp
lincom c.IPV#c.NvC*-1 + c.IPV#c.NvS*1, irr  // No/Yes IPV differ btw CogR/Supp


display "STATA Link=Log Dist=Negative Binomial Model using nbreg"
display "nbreg gives LRT for scale factor that distinguishes NB from Poisson"
nbreg aggr c.IPV c.NvC c.NvS c.IPV#c.NvC c.IPV#c.NvS, nolog
display "-2LL=" e(ll)*-2  // Print -2LL for model

display "STATA Link=Log Dist=Negative Binomial Model using glm"
display "glm gives conditional fit, scale factor estimated by ML"
glm aggr c.IPV c.NvC c.NvS c.IPV#c.NvC c.IPV#c.NvS, ml link(log) family(nbinomial ml) nolog
display "-2LL= " e(ll)*-2  // Print -2LL for model
// DF=5 Multiv Wald Test of Model R2
test (c.IPV=0)(c.NvC=0)(c.NvS=0)(c.IPV#c.NvC=0)(c.IPV#c.NvS=0)
// DF=2 Multiv Wald Test of Interaction
test (c.IPV#c.NvC=0)(c.IPV#c.NvS=0)
// Yhat cell means in log count model scale per condition
margins, at(c.IPV=(0(1)1) c.NvC=0 c.NvS=0) predict(xb) // None
margins, at(c.IPV=(0(1)1) c.NvC=1 c.NvS=0) predict(xb) // CogR
margins, at(c.IPV=(0(1)1) c.NvC=0 c.NvS=1) predict(xb) // Supp
// Y hat cell means in expected count data scale per ercond
margins, at(c.IPV=(0(1)1) c.NvC=0 c.NvS=0) // None
margins, at(c.IPV=(0(1)1) c.NvC=1 c.NvS=0) // CogR
margins, at(c.IPV=(0(1)1) c.NvC=0 c.NvS=1) // Supp
// Simple slopes of IPV per condition
lincom c.IPV*1 + c.IPV#c.NvC*0 + c.IPV#c.NvS*0  // No vs Yes IPV: None
lincom c.IPV*1 + c.IPV#c.NvC*1 + c.IPV#c.NvS*0  // No vs Yes IPV: CogR
lincom c.IPV*1 + c.IPV#c.NvC*0 + c.IPV#c.NvS*1  // No vs Yes IPV: Supp
// Simple slopes of condition per IPV
lincom c.NvC*1  + c.NvS*0 + c.IPV#c.NvC*0  + c.IPV#c.NvS*0  // None vs CogR: IPV=No
lincom c.NvC*0  + c.NvS*1 + c.IPV#c.NvC*0  + c.IPV#c.NvS*0  // None vs Supp: IPV=No
lincom c.NvC*-1 + c.NvS*1 + c.IPV#c.NvC*0  + c.IPV#c.NvS*0  // CogR vs Supp: IPV=No
lincom c.NvC*1  + c.NvS*0 + c.IPV#c.NvC*1  + c.IPV#c.NvS*0  // None vs CogR: IPV=Yes
lincom c.NvC*0  + c.NvS*1 + c.IPV#c.NvC*0  + c.IPV#c.NvS*1  // None vs Supp: IPV=Yes
lincom c.NvC*-1 + c.NvS*1 + c.IPV#c.NvC*-1 + c.IPV#c.NvS*1  // CogR vs Supp: IPV=Yes
// Simple slopes for interaction contrasts
lincom c.IPV#c.NvC*1  + c.IPV#c.NvS*0  // No/Yes IPV differ btw None/CogR
lincom c.IPV#c.NvC*0  + c.IPV#c.NvS*1  // No/Yes IPV differ btw None/Supp
lincom c.IPV#c.NvC*-1 + c.IPV#c.NvS*1  // No/Yes IPV differ btw CogR/Supp
// Save predicted counts per real person to dataset
predict predcount
corr predcount aggr     // Get of correlation of pred count with aggr
display "R2=" r(rho)^2  // Print R2 relative to empty model

 
display "STATA Link=Log Dist=Negative Binomial Model"
display "Request Incidence-Rate Ratios (via eform or irr)"
glm aggr c.IPV c.NvC c.NvS c.IPV#c.NvC c.IPV#c.NvS, eform ml link(log) family(nbinomial ml) nolog
// Simple slopes of IPV per condition
lincom c.IPV*1 + c.IPV#c.NvC*0 + c.IPV#c.NvS*0, irr  // No vs Yes IPV: None
lincom c.IPV*1 + c.IPV#c.NvC*1 + c.IPV#c.NvS*0, irr  // No vs Yes IPV: CogR
lincom c.IPV*1 + c.IPV#c.NvC*0 + c.IPV#c.NvS*1, irr  // No vs Yes IPV: Supp
// Simple slopes of condition per IPV
lincom c.NvC*1  + c.NvS*0 + c.IPV#c.NvC*0  + c.IPV#c.NvS*0, irr  // None vs CogR: IPV=No
lincom c.NvC*0  + c.NvS*1 + c.IPV#c.NvC*0  + c.IPV#c.NvS*0, irr  // None vs Supp: IPV=No
lincom c.NvC*-1 + c.NvS*1 + c.IPV#c.NvC*0  + c.IPV#c.NvS*0, irr  // CogR vs Supp: IPV=No
lincom c.NvC*1  + c.NvS*0 + c.IPV#c.NvC*1  + c.IPV#c.NvS*0, irr  // None vs CogR: IPV=Yes
lincom c.NvC*0  + c.NvS*1 + c.IPV#c.NvC*0  + c.IPV#c.NvS*1, irr  // None vs Supp: IPV=Yes
lincom c.NvC*-1 + c.NvS*1 + c.IPV#c.NvC*-1 + c.IPV#c.NvS*1, irr  // CogR vs Supp: IPV=Yes
// Simple slopes for interaction contrasts
lincom c.IPV#c.NvC*1  + c.IPV#c.NvS*0, irr  // No/Yes IPV differ btw None/CogR
lincom c.IPV#c.NvC*0  + c.IPV#c.NvS*1, irr  // No/Yes IPV differ btw None/Supp
lincom c.IPV#c.NvC*-1 + c.IPV#c.NvS*1, irr  // No/Yes IPV differ btw CogR/Supp
 
 
display "STATA Zero-Inflated Poisson Model"
display "Only intercept in zero-inflation model (predict logit of extra zero)"
zip aggr c.IPV c.NvC c.NvS c.IPV#c.NvC c.IPV#c.NvS, inflate(_cons) nolog
display "-2LL= " e(ll)*-2  // Print -2LL for model
nlcom 1/(1+exp(-1* _b[inflate:_cons])) // Probability of extra 0 


// *** STATA BLEW UP!!
// display "STATA Zero-Inflated Negative Binomial Predicting Aggressive Verbalizations"
// display "Only intercept in zero-inflation model (predict logit of extra zero)"
// zinb aggr c.IPV c.NvC c.NvS c.IPV#c.NvC c.IPV#c.NvS, inflate(_cons)
// display "-2LL=" e(ll)*-2  // Print -2LL for model
// nlcom 1/(1+exp(-1* _b[inflate:_cons])) // Probability of extra 0 


// Run search below, then install from window that pops up
search countfit

display "STATA Countfit to Compare Fit of Alternative Count Model Distributions"
display "prm=Poisson, nbreg=Negative Binomial, zip=Zero-Inflated Poisson"
display "Results suggest NegBin fits best"
countfit aggr c.IPV c.NvC c.NvS c.IPV#c.NvC c.IPV#c.NvS, prm nbreg zip replace
graph export "$filesave\STATA Predicted Counts from Countfit.png", replace


display "STATA Heterogeneous Negative Binomial Model"
display "lnalpha gives linear model to predict log of scale factor"
gnbreg aggr c.IPV c.NvC c.NvS c.IPV#c.NvC c.IPV#c.NvS, ///
       lnalpha(c.IPV c.NvC c.NvS c.IPV#c.NvC c.IPV#c.NvS) nolog
display "-2LL= " e(ll)*-2  // Print -2LL for model


// Close external file of saved results  
log close Example3



